-- Buzzbox Core

local addonName, addon = ...
local Buzzbox = addon

local buzzboxLDB = LibStub("LibDataBroker-1.1"):NewDataObject("Buzzbox", {
	type = "data source",
	text = "BB (Off)",
	icon = "Interface\\Icons\\Inv_jewelcrafting_dragonseye04", -- must be registered on initialize
	OnClick = function(self, button) addon:mmbutton(button) end,
	OnTooltipShow = function(tooltip)
		tooltip:SetText(""..Buzzbox.db.profile.devicename.."")
		tooltip:AddLine("Currently tuned to "..Buzzbox.db.profile.tfreq.."kHz.\nRight click to turn the Buzzbox on and off.\nLeft click to activate and deactivate the microphone.\nMiddle click to bring up the options panel.", 1, 1, 1)
		tooltip:Show()
	end,
})

local icon = LibStub("LibDBIcon-1.0")

--	--------------------
--			GUI functions
--	--------------------

function addon:ToggleMinimapButton()
	self.db.profile.minimap.hide = not self.db.profile.minimap.hide
	if self.db.profile.minimap.hide then
		icon:Hide("Buzzbox")
	else
		icon:Show("Buzzbox")
	end
end

function addon:GetMinimapIcon()
  if Buzzbox.status == 0 then
    return "Interface\\Icons\\Inv_jewelcrafting_dragonseye04"
  else
    if Buzzbox.mic_status == 1 then
      return "Interface\\Icons\\Inv_jewelcrafting_dragonseye05"
    else
      return "Interface\\Icons\\Inv_jewelcrafting_dragonseye03"
    end
  end
end

function addon:GetLDBText()
	if Buzzbox.status == 0 then
    return "BB (Off)"
  else
    if Buzzbox.mic_status == 1 then
      return "BB - "..self.db.profile.tfreq.."kHz (Mic On)"
    else
      return "BB - "..self.db.profile.tfreq.."kHz (On)"
    end
  end
end

function addon:mmbutton(button, down)
  if button == "LeftButton" then
  	addon:ToggleMic()
  elseif button == "RightButton" then
    addon:ToggleStatus()
	elseif button == "MiddleButton" then
		addon:ShowConfig()
  end
end

function addon:UpdateLDB()
	buzzboxLDB.text = addon:GetLDBText()
  buzzboxLDB.icon = addon:GetMinimapIcon()
end

function addon:ShowConfig()
	InterfaceOptionsFrame_OpenToCategory("Buzzbox")
	InterfaceOptionsFrame_OpenToCategory("Buzzbox")
end

--	------------------------
--			Device functions
--	------------------------

function addon:ToggleMic()
  if Buzzbox.status == 1 then
    if Buzzbox.mic_status == 0 then
      Buzzbox.mic_status = 1
			if Buzzbox.db.profile.autoem == false then
      	Buzzbox:Print("Transmitting...")
			elseif Buzzbox.db.profile.autoem == true and Buzzbox.db.profile.micsens == 1 then
				SendChatMessage("presses a button on "..self.sex.." "..self.db.profile.devicename.." and holds it close to "..self.sex.." mouth.","EMOTE",nil);
			elseif Buzzbox.db.profile.autoem == true and Buzzbox.db.profile.micsens == 2 then
				SendChatMessage("presses a button on "..self.sex.." "..self.db.profile.devicename.." and holds it up.","EMOTE",nil);
			end
    else
      Buzzbox.mic_status = 0
			if Buzzbox.db.profile.autoem == false then
      	Buzzbox:Print("Microphone muted. Transmission stopped.")
			else
				SendChatMessage("lowers "..self.sex.." "..self.db.profile.devicename..".","EMOTE",nil);
			end
    end
  else
    Buzzbox:Print("\124cFFFF8040*click* *click* *click*\124r Nothing... maybe try turning it on first?")
  end
	addon:UpdateLDB()
end

function addon:ToggleStatus()
  if Buzzbox.status == 0 then
    Buzzbox.status = 1
    Buzzbox:Print("\124cFFFF8040*click*\124r You turn on the device.")
  else
    Buzzbox.status = 0
    Buzzbox:Print("You turn the device off. \124cFFFF8040*click*")
  end
	addon:UpdateLDB()
end

function addon:GetSenderRPName(words, wordc)
	local name = words[4]
	for i=5,wordc-1,1 do
		name = name.." "..words[i]
  end
	return name
end

function addon:GetEncryptionKey()
  if self.db.profile.encryptionopt ~= false then
    return self.db.profile.encryptionKey
  else
    return "0"
  end
end

function addon:PrintMessage(message, sender)
	if Buzzbox.db.profile.volume == 1 then
	  if Buzzbox.db.profile.sendernameopt == 1 then
	    SendChatMessage("'s "..self.db.profile.devicename..":"..message.." (( Sent by "..sender.." ))","EMOTE",nil);
	  elseif Buzzbox.db.profile.sendernameopt == 2 then
	    SendChatMessage("'s "..self.db.profile.devicename.." emits "..sender.."'s voice:"..message.."","EMOTE",nil);
	  else
	    SendChatMessage("'s "..self.db.profile.devicename..":"..message.."","EMOTE",nil);
	  end
	elseif Buzzbox.db.profile.volume == 2 then
		if Buzzbox.db.profile.sendernameopt == 1 then
			DEFAULT_CHAT_FRAME:AddMessage("["..self.db.profile.devicename.."] whispers:"..message.." (( Sent by "..sender.." ))",1,0.5,1)
	  elseif Buzzbox.db.profile.sendernameopt == 2 then
	    DEFAULT_CHAT_FRAME:AddMessage("["..self.db.profile.devicename.."] whispers with "..sender.."'s voice:"..message.."",1,0.5,1)
	  else
	    DEFAULT_CHAT_FRAME:AddMessage("["..self.db.profile.devicename.."] whispers:"..message.."",1,0.5,1)
	  end
	end
end

--	----------------
--			Snippets
--	----------------

function addon:InsertCrackles(message)
  local msg = ""
    for i, word in ipairs{strsplit(" ", message)} do
        if word ~= "" then
            if math.random() <= math.pow(1/Buzzbox.maxdistance,10) * math.pow(Buzzbox.distance,10) then
                msg = msg.." *"..Buzzbox.crackles[math.random(#Buzzbox.crackles)].."*";
            else msg = msg.." "..word;
            end
        end
    end
  return msg
end

function addon:MicrophoneActive()
  return self.mic_status == 1 and self.status == 1
end

function addon:RemoveServer(name)
  return name:match("^([^%-]+)%-(.+)$")
end

function addon:GetRPName()
  if TRP3_API ~= nil then
    return TRP3_API.r.name("player")
  else
    return ""
  end
end

function addon:BreakStringIntoWords(arg1)
  local words = {}
  local word_count = 0
  for word in arg1:gmatch("%S+") do
    table.insert(words, word)
    word_count = word_count+1
  end
  return words, word_count
end

function addon:OnInitialize()
  --  ----------------------
  --      Addon's Options
  --  ----------------------
  self.db = LibStub("AceDB-3.0"):New("BuzzboxDB", self.defaults)
  self.ConfigRegistry = LibStub("AceConfig-3.0"):RegisterOptionsTable("Buzzbox", self.Options, {"buzzbox", "bb"})
  self.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("Buzzbox", "Buzzbox")

  icon:Register("Buzzbox", buzzboxLDB, self.db.profile.minimap)

  --  ------------------------
  --      Needed Variables
  --  ------------------------
  self.rpname = nil -- TRP name
	self.id = nil -- channel
  self.pname = UnitName("player") -- player's name
  self.crackles = { "Bzz", "Tzzt", "Bzzt", "Tzz", "Crackle", "Chrrzz", "Khrrzz", "Crrhzz", "Static" };
  self.mic_status = 0;
  self.status = 0;
	self.maxdistance = 1500;
	if UnitSex("player") == 2 then
	    self.sex = "his"
	elseif UnitSex("player") == 3 then
	    self.sex = "her"
	end

  --  ------------------------
  --      Registered Events
  --  ------------------------
  self:RegisterEvent("CHAT_MSG_SAY")
  self:RegisterEvent("CHAT_MSG_CHANNEL")
	self:RegisterEvent("CHAT_MSG_YELL")

  -- Welcome message
  self:Print("Addon loaded. Buzzbox currently turned off. To start receiving messages, type </bb io> or go to Interface -> Addons and select the Buzzbox options to enable it.")

  -- Join xtensiontooltip2 if not already in
  local frameMA = CreateFrame("FRAME");
  frameMA:RegisterEvent("PLAYER_ENTERING_WORLD");
  function frameMA:OnEvent(event)
    JoinChannelByName("xtensionxtooltip2", nil);
  end
  frameMA:SetScript("OnEvent", frameMA.OnEvent);
end
