local addonName, addon = ...
local Buzzbox = addon

local yellow = "|cffffd100%s|r"
local orange = "|cffff8040%s|r"

addon.Options = {
  type = "group",
  name = "Buzzbox",

  childGroups = "tree",
  handler = Buzzbox,

  args = {

    usage_commands = {

      type = "group",
      name = "Device",
      desc = "Type /bb <command> to quickly do the following operations",
      inline = true,
      order = 0,
      args = {

        mic = {
          order = 2,
          name = "Mic",
          width = "half",
          desc = "Mute / Unmute the microphone",
          type = "execute",
          func = "ToggleMic"
        },

        io = {
          order = 1,
          name = "I/O",
          width = "half",
          desc = "Turn the device on and off",
          type = "execute",
          func = "ToggleStatus"
        },

        freq = {
          order = 3,
          name = "Tuning Frequency",
          desc = "Set the frequency your device is tuned to",
          type = "range",
          min = 100,
          max = 999,
          step = 1,
          set = function(info,val)
            Buzzbox.db.profile.tfreq = val
            addon:UpdateLDB()
          end,
          get = function(info) return Buzzbox.db.profile.tfreq end
        },

        vol = {
          order = 4,
          name = "Speaker Vol.",
          desc = "Set the volume of your device",
          width = "half",
          type = "select",
          style = "dropdown",
          values = {"Loud","Whisper","Muted"},
          set = function(info,val) Buzzbox.db.profile.volume = val end,
          get = function(info) return Buzzbox.db.profile.volume end
        },

        sens = {
          order = 5,
          name = "Mic Sens.",
          desc = "Set whose voice the microphone can pick up.\n"..orange:format("Be careful when using \"Everyone\" as WoW may temporarily block your sent messages due to excessive spamming."),
          width = "half",
          type = "select",
          cmdHidden = true,
          style = "dropdown",
          values = {"Self",orange:format("Everyone")},
          set = function(info,val) Buzzbox.db.profile.micsens = val end,
          get = function(info) return Buzzbox.db.profile.micsens end
        },
      }
    },

    preferences = {

      type = "group",
      name = "Preferences",
      cmdHidden = true,
      order = 1,
      args = {

        sendernameopt = {
          type = "toggle",
          name = "Show IC names",
          desc = "Toggle between showing IC names or actual, OOC, character names",
          width = "full",
          order = 2,
          set = function(info, val) Buzzbox.db.profile.senderRPnameopt = val; end,
          get = function(info) return Buzzbox.db.profile.senderRPnameopt end
        },

        auto_emote = {
          type = "toggle",
          name = "Auto-emote",
          desc = "Allow the Buzzbox to emote whenever you speak into it.\n"..orange:format("Be mindful and turn this off if you are going to toggle the microphone excessively. Do not spam the chat!"),
          width = "full",
          order = 3,
          set = function(info, val) Buzzbox.db.profile.autoem = val; end,
          get = function(info) return Buzzbox.db.profile.autoem end
        },

        message_noise = {
          type = "toggle",
          name = "Low signal crackles",
          desc = "Enable / Disable crackles in low signal messages",
          width = "full",
          order = 1,
          set = function(info, val) Buzzbox.db.profile.noiseopt = val; end,
          get = function(info) return Buzzbox.db.profile.noiseopt end
        },

        encryption = {
          type = "toggle",
          name = "Encryption",
          desc = "Enable end-to-end encryption",
          order = 5,
          set = function(info, val) Buzzbox.db.profile.encryptionopt = val; end,
          get = function(info) return Buzzbox.db.profile.encryptionopt end
        },

        minimap_button = {
          type = "toggle",
          name = "Hide minimap button",
          desc = "Hide the minimap button\nAs an alternative, you can use Buzzbox with a Data Broker AdddOn",
          width = "full",
          order = 4,
          set = function(info, val) Buzzbox:ToggleMinimapButton(); end,
          get = function(info) return Buzzbox.db.profile.minimap.hide end
        },

        encryption_key = {
          type = "input",
          name = "Encryption Key",
          desc = "Enter a key to encrypt and decrypt messages with",
          order = 6,
          set = function(info, val) Buzzbox.db.profile.encryptionKey = val:gsub("%s+",""); end,
          get = function(info) return Buzzbox.db.profile.encryptionKey end,
          validate =
          function(info, val)
            if string.len(val)>8 then
              Buzzbox:Print("Your key cannot be more than 8 characters.")
              return "Error"
            elseif val=="" then
              Buzzbox.db.profile.encryptionKey = "0"
              return "Error"
            else
              return true
            end
          end
        }
      }
    },

    customization = {

      type = "group",
      name = "Customization",
      cmdHidden = true,
      order = 3,
      args = {
        drange = {
          order = 3,
          name = "Range",
          desc = "How far away (in yards) your device can send signals\nDefault and recommended value: 1500",
          type = "range",
          min = 0,
          max = 10000,
          step = 100,
          set = function(info,val) Buzzbox.db.profile.maxrange = val end,
          get = function(info) return Buzzbox.db.profile.maxrange end
        },
        devicename = {
          type = "input",
          name = "Device name",
          desc = "The device name that will be used in emotes",
          order = 1,
          set = function(info, val) Buzzbox.db.profile.devicename = val; end,
          get = function(info) return Buzzbox.db.profile.devicename end,
          validate =
          function(info, val)
            if string.len(val)>16 then
              Buzzbox:Print("Name cannot be more than 16 characters.")
              return "Error"
            elseif val=="" then
              Buzzbox.db.profile.devicename = "Buzzbox"
              return "Error"
            else
              return true
            end
          end
        },
        sender_name = {
          name = "Sender's name display style",
          desc = "Select the way you want the sender's name to be displayed",
          type = "select",
          style = "dropdown",
          order = 2,
          values = {"(( Sent by ... ))","Part of the emote","Do not show sender's name"},
          set = function(info, val) Buzzbox.db.profile.sendernameopt = val; end,
          get = function(info) return Buzzbox.db.profile.sendernameopt end
        },
      },
    },
    about = {
      type = "group",
      name = "About",
      cmdHidden = true,
      order = -1,
      args = {
        title = {
          type = "description",
          name = yellow:format("Title: ")..addon.title,
          order = 1,
        },
        version = {
          type = "description",
          name = yellow:format("Version: ")..addon.version,
          order = 2,
        },
        author = {
          type = "description",
          name = yellow:format("Author: ")..addon.author,
          order = 3,
        },
        license = {
          type = "description",
          name = yellow:format("License: ").."GNU General Public License version 3 (GPLv3)",
          order = 4,
        },
        website = {
          type = "description",
          name = yellow:format("Project Page: ").."https://www.wowace.com/projects/buzzbox",
          order = 4,
        },
      }
    },
  }
}
